#!/usr/bin/env python

import os, os.path, subprocess, debconf

from RestrictedManager.core import DefaultHandler, MANAGER_ROOT, notify_reboot_required

from gettext import gettext as _

class XorgDriverHandler(DefaultHandler):
    """Abstract class for a restricted X.org graphics driver."""

    def __init__(self, module, driver_package, xorg_driver, extra_conf_options,
        free_driver, add_modules, remove_modules):
        """Build graphics driver handler for a particular concrete kernel
        module."""

        DefaultHandler.__init__(self, module)
        self.driver_package = driver_package
        self.xorg_driver = xorg_driver
        self.extra_xorg_driver_options = extra_conf_options
        self.free_driver = free_driver
        self.add_modules = add_modules
        self.remove_modules = remove_modules

    def can_change(self):
        """Check whether xorg.conf has been customized."""

        checksum_file = "/var/lib/x11/xorg.conf.md5sum"
        if os.path.exists(checksum_file) and subprocess.call(["md5sum",
            "--status", "-c", checksum_file]) != 0:
            return _("Your /etc/x11/xorg.conf has been manually changed. Sorry, "
                "restricted-manager cannot be used with customized "
                "configurations.\n\n"
                "Call \"sudo dpkg-reconfigure -phigh xserver-xorg\""
                "to switch back to automatic configuration handling.")

        return None

    def is_enabled(self):
        """Returns True if the module is enabled."""
        dc = debconf.DebconfCommunicator("restricted-manager")
        driver = dc.get("xserver-xorg/config/device/driver")
        dc.shutdown()

        return driver == self.xorg_driver and self.package_installed(self.driver_package)

    def enable(self):
        """Enable the module."""

        DefaultHandler.enable(self)

        if not self.package_installed(self.driver_package):
            self.install_package(self.driver_package)

        # do not mangle xorg.conf if package installation has been aborted
        if not self.package_installed(self.driver_package):
            return

        dc = debconf.DebconfCommunicator("restricted-manager")
        dc.capb('escape')

        # remember the previous driver
        cur_driver = dc.get("xserver-xorg/config/device/driver")
        open(os.path.join(MANAGER_ROOT, self.name + ".olddriver"), "w").write(cur_driver)

        dc.set("xserver-xorg/config/device/driver", self.xorg_driver)
        dc.fset("xserver-xorg/config/device/driver", "seen", "true")

        # don't let xserver-xorg clobber our selected driver and previous
        # settings
        dc.set("xserver-xorg/autodetect_video_card", "false")
        dc.fset("xserver-xorg/autodetect_video_card", "seen", "true")
        auto_monitor = dc.get("xserver-xorg/autodetect_monitor")
        dc.set("xserver-xorg/autodetect_monitor", "false")
        dc.fset("xserver-xorg/autodetect_monitor", "seen", "true")
        auto_mouse = dc.get("xserver-xorg/autodetect_mouse")
        dc.set("xserver-xorg/autodetect_mouse", "false")
        dc.fset("xserver-xorg/autodetect_mouse", "seen", "true")

        modules = dc.get("xserver-xorg/config/modules").split(", ")
        for m in self.remove_modules:
            while m in modules:
                modules.remove(m)
        for m in self.add_modules:
            if m not in modules:
                modules.append(m)
        dc.set("xserver-xorg/config/modules", ", ".join(modules))
        dc.fset("xserver-xorg/config/modules", "seen", "true")

        dc.set("xserver-xorg/config/device/extra_options",
            self.extra_xorg_driver_options)
        dc.fset("xserver-xorg/config/device/extra_options", "seen", "true")

        dc.shutdown()

        log = open('/var/log/restricted-manager.log', 'a')
        print >> log, '*** Enabling', self.name, ': debconf dump before dpkg-reconfigure:'
        log.flush()
        subprocess.call(['grep', '-A', '5', '^Name: xserver-xorg',
            '/var/cache/debconf/config.dat'], stdout=log)
        print >> log, '*** Enabling', self.name, ': xorg.conf before dpkg-reconfigure:'
        log.flush()
        subprocess.call(['grep', '-A', '5', '^Section "Device"',
            '/etc/X11/xorg.conf'], stdout=log)

        subprocess.call(['/usr/sbin/dpkg-reconfigure', '-fnoninteractive', '-pcritical',
            'xserver-xorg'])

        # restore our clobbered autodetection values
        dc = debconf.DebconfCommunicator("restricted-manager")
        dc.set("xserver-xorg/autodetect_monitor", auto_monitor)
        dc.set("xserver-xorg/autodetect_mouse", auto_mouse)
        dc.shutdown()

        print >> log, '*** Enabling', self.name, ': debconf dump after dpkg-reconfigure:'
        log.flush()
        subprocess.call(['grep', '-A', '5', '^Name: xserver-xorg',
            '/var/cache/debconf/config.dat'], stdout=log)
        print >> log, '*** Enabling', self.name, ': xorg.conf after dpkg-reconfigure:'
        log.flush()
        subprocess.call(['grep', '-A', '5', '^Section "Device"',
            '/etc/X11/xorg.conf'], stdout=log)

        log.close()

        notify_reboot_required()

    def disable(self):
        """Disable the module."""

        DefaultHandler.disable(self)

        if self.package_installed(self.driver_package):
            self.remove_package(self.driver_package)

        # try to get the previous driver
        try:
            p = os.path.join(MANAGER_ROOT, self.name + ".olddriver")
            free_driver = open(p).read().strip()
            os.unlink(p)
        except IOError:
            free_driver = self.free_driver

        dc = debconf.DebconfCommunicator("restricted-manager")
        dc.set("xserver-xorg/config/device/driver", free_driver)
        dc.fset("xserver-xorg/config/device/driver", "seen", "true")

        # let xserver-xorg override our free driver if it wants to
        dc.set("xserver-xorg/autodetect_video_card", "true")
        dc.fset("xserver-xorg/autodetect_video_card", "seen", "true")
        auto_monitor = dc.get("xserver-xorg/autodetect_monitor")
        dc.set("xserver-xorg/autodetect_monitor", "false")
        dc.fset("xserver-xorg/autodetect_monitor", "seen", "true")
        auto_mouse = dc.get("xserver-xorg/autodetect_mouse")
        dc.set("xserver-xorg/autodetect_mouse", "false")
        dc.fset("xserver-xorg/autodetect_mouse", "seen", "true")

        modules = dc.get("xserver-xorg/config/modules").split(", ")
        for m in self.add_modules:
            while m in modules:
                modules.remove(m)
        for m in self.remove_modules:
            if m not in modules:
                modules.append(m)
        dc.set("xserver-xorg/config/modules", ", ".join(modules))
        dc.fset("xserver-xorg/config/modules", "seen", "true")

        dc.set("xserver-xorg/config/device/extra_options", "")
        dc.fset("xserver-xorg/config/device/extra_options", "seen", "true")
        dc.shutdown()

        subprocess.call(['/usr/sbin/dpkg-reconfigure', '-fnoninteractive', '-pcritical',
            'xserver-xorg'])

        # restore our clobbered autodetection values
        dc = debconf.DebconfCommunicator("restricted-manager")
        dc.set("xserver-xorg/autodetect_monitor", auto_monitor)
        dc.set("xserver-xorg/autodetect_mouse", auto_mouse)
        dc.shutdown()

        notify_reboot_required()
